;;; notify.el ---

;; Copyright (C) 1995 Noah S. Friedman

;; Author: Noah Friedman <friedman@prep.ai.mit.edu>
;; Maintainer: friedman@prep.ai.mit.edu
;; Keywords: extensions
;; Created: 1995-03-18

;;; $Id: zenirc-notify.el,v 1.0 1995/03/23 04:43:55 friedman Exp $

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, you can either send email to this
;; program's maintainer or write to: The Free Software Foundation,
;; Inc.; 675 Massachusetts Avenue; Cambridge, MA 02139, USA.

;;; Commentary:

;; There is presently no coherence to this code.  I'm just gathering
;; things out of zenirc.el for later assembly.

;;; Code:

(defvar zenirc-timer-hook '(zenirc-notify-timer)
  "Timer hook variable.")

(defvar zenirc-notify-interval '(0 60)
  "Time between ISON's sent to server.")

(defvar zenirc-last-notify '(0 0)
  "Time previous ISON sent.")

(defvar zenirc-command-ison-hook '(zenirc-command-ison))

(defvar zenirc-command-notify-hook '(zenirc-command-notify))


;; used with notify code to see if we did /ison or not
(defvar zenirc-manual-ison nil)

;; previous people you recieved notification of
(defvar zenirc-previous-ison nil)

;     (s303 . "[info] Currently wasting time: %s\n") ; ison repl
;     (notify_list . "[info] Your current notify list: %s\n")
;     (notify_on . "[info] detected %s wasting time.\n")
;     (notify_off . "[info] detected that %s stopped wasting time.\n")


;; /notify handler
;;
;; *** NOTE ***
;; this is also called from the zenirc event handling code
;;
;; FIXME: this code is fugly.
(defun zenirc-command-notify (proc parsedcmd)
  (let ((arg (cdr parsedcmd)))
    (if (and parsedcmd (not (string= "" arg)))
	(progn
	  (if (string= "list" arg)
	      (zenirc-message proc 'notify_list
                              (mapconcat 'identity zenirc-notify-list " ")))
	  (if (string-match "+" (substring arg
					   (- (length arg)) 1))
	      (if (not (member (substring arg 1)
                               zenirc-notify-list))
		  (setq zenirc-notify-list (cons (substring arg 1)
						 zenirc-notify-list))))
	  (if (string-match "-" (substring arg
					   (- (length arg)) 1))
	      (setq zenirc-notify-list (delete
					(substring arg 1)
					zenirc-notify-list)))))
    (if (and (or (string= "" arg) (not parsedcmd))
	     zenirc-notify-list)
	(process-send-string
	 proc
	 (concat "ISON " (mapconcat 'identity zenirc-notify-list " ") "\n")))))


(defun zenirc-notify-timer (proc now)
  "Call zenirc-command-notify-hook with arguments that cause it to send an
ISON message to the server. This is used to notice when people in the notify
list have come on or off of IRC."
  (if (zenirc-time< zenirc-notify-interval
		    (zenirc-time-diff now zenirc-last-notify))
      (progn
	(zenirc-run-hook 'zenirc-command-notify-hook proc '("notify" . ""))
	(setq zenirc-last-notify now))))

;; /ison nick1 [nick2 [nick3...]]
;;
;; stupid way to make /notify code understand that you want a ison reply
;; FIXME!
(defun zenirc-command-ison (proc parsedcmd)
  (process-send-string proc
		       (concat "ISON " (cdr parsedcmd) "\n"))
  (setq zenirc-manual-ison 1))

;; 303 - Ison reply - RPL_ISON ":[<nick> {<space><nick>}]"
(defun zenirc-server-303 (proc parsedmsg)
  (let* ((ison-list (zenirc-parse-words (aref parsedmsg 3)))
	 (temp-list ison-list))
    (if zenirc-manual-ison
	; we did /ison and don't want to notify code to execute
	(progn
	  (zenirc-message proc 's303 (aref parsedmsg 3))
	  (setq zenirc-manual-ison nil))
      (if (string= (aref parsedmsg 3) "")
	; nobody on our list is on
	(setq zenirc-previous-ison nil)
      ; someone on our notify list is on
      (while temp-list
	(if (not (member (car temp-list)
                         zenirc-previous-ison))
	    (zenirc-message proc 'notify_on (car temp-list)))
	(setq temp-list (cdr temp-list)))
      (while zenirc-previous-ison
	(if (not (member (car zenirc-previous-ison)
                         ison-list))
	    (zenirc-message proc 'notify_off (car zenirc-previous-ison)))
	(setq zenirc-previous-ison (cdr zenirc-previous-ison)))
      (setq zenirc-previous-ison ison-list)))))


(defun zenirc-notify-timer (proc now)
  "Call zenirc-command-notify-hook with arguments that cause it to send an
ISON message to the server. This is used to notice when people in the notify
list have come on or off of IRC."
  (if (zenirc-time< zenirc-notify-interval
		    (zenirc-time-diff now zenirc-last-notify))
      (progn
	(zenirc-run-hook 'zenirc-command-notify-hook proc '("notify" . ""))
	(setq zenirc-last-notify now))))

;;; notify.el ends here

